@@ -1,3 +1,3 @@
-from .batch_job import BatchJob  # noqa: F401
+from .batch_job import BatchJob, BatchJobNoEFS  # noqa: F401
 from .cli_run_s3upload import miniwdl_run_s3upload  # noqa: F401
 from .cli_submit import miniwdl_submit_awsbatch  # noqa: F401
